package me.km.blocks;

import java.util.Random;
import javax.annotation.Nullable;
import me.km.items.ModItems;
import static net.minecraft.block.Block.NULL_AABB;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.IShearable;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class BlockResistantTallGrass extends BlockBase implements net.minecraftforge.common.IPlantable, IShearable
{
    protected static final AxisAlignedBB TALL_GRASS_AABB = new AxisAlignedBB(0.1d, 0.0d, 0.1d, 0.9d, 0.8d, 0.9d);
    
    public BlockResistantTallGrass(String name, String local) 
    {
        super(Material.VINE, name, local);
        super.setCreativeTab(CreativeTabs.DECORATIONS);
        super.setSoundType(SoundType.PLANT);
        super.setHardness(0);
    }

    @Override
    public AxisAlignedBB getBoundingBox(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return TALL_GRASS_AABB;
    }

    @Nullable
    @Override
    public AxisAlignedBB getCollisionBoundingBox(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return NULL_AABB;
    }

    @Override
    public boolean isOpaqueCube(IBlockState state)
    {
        return false;
    }

    @Override
    public boolean isFullCube(IBlockState state)
    {
        return false;
    }

    @Override
    public net.minecraftforge.common.EnumPlantType getPlantType(IBlockAccess world, BlockPos pos)
    {
        return net.minecraftforge.common.EnumPlantType.Plains;
    }

    @Override
    public IBlockState getPlant(net.minecraft.world.IBlockAccess world, BlockPos pos)
    {
        IBlockState state = world.getBlockState(pos);
        if(state.getBlock() != this) 
        {
            return getDefaultState();
        }
        return state;
    }

    @SideOnly(Side.CLIENT)
    @Override
    public BlockRenderLayer getBlockLayer()
    {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    public BlockFaceShape getBlockFaceShape(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @Override
    public boolean isReplaceable(IBlockAccess worldIn, BlockPos pos)
    {
        return true;
    }

    @Override
    public Item getItemDropped(IBlockState state, Random rand, int fortune)
    {
        return null;
    }

    @Override
    public int quantityDroppedWithBonus(int fortune, Random random)
    {
        return 1 + random.nextInt(fortune * 2 + 1);
    }

    @Override
    public void harvestBlock(World w, EntityPlayer p, BlockPos pos, IBlockState state, TileEntity te, ItemStack stack)
    {
        if(!w.isRemote && stack.getItem() == Items.SHEARS)
        {
            p.addStat(StatList.getBlockStats(this));
            spawnAsEntity(w, pos, new ItemStack(ModBlocks.tallGrass, 1));
        }
        else
        {
            super.harvestBlock(w, p, pos, state, te, stack);
        }
    }

    @Override 
    public boolean isShearable(ItemStack item, IBlockAccess world, BlockPos pos)
    {
        return true; 
    }
    
    @Override
    public NonNullList<ItemStack> onSheared(ItemStack item, IBlockAccess world, BlockPos pos, int fortune)
    {
        return NonNullList.withSize(1, new ItemStack(ModBlocks.tallGrass, 1));
    }
    
    @Override
    public NonNullList<ItemStack> getDrops(IBlockAccess w, BlockPos pos, IBlockState state, int fortune)
    {
        int rand = RANDOM.nextInt(8);
        if(rand <= 3)
        {
            return NonNullList.withSize(1, new ItemStack(ModItems.hayBundle));
        }
        else if(RANDOM.nextInt(8) == 4) 
        {
            ItemStack seed = net.minecraftforge.common.ForgeHooks.getGrassSeed(RANDOM, fortune);
            if(!seed.isEmpty())
            {
                return NonNullList.withSize(1, seed);
            }
        }
        return NonNullList.create();
    }
}
