package me.km.playerbank;

import java.util.HashMap;
import java.util.UUID;
import me.kcm.events.PlayerConnectionEvent;
import me.km.KajetansMod;
import me.km.module.Module;
import me.km.databank.DataBank;
import me.km.scoreboard.ScoreboardUtils;
import me.km.snuviscript.ScriptEvents;
import me.km.world.ModDimensions;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.text.TextFormatting;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.PlayerEvent;

public class PlayerManager extends Module
{
    private final PlayerBank bank;
    private final HashMap<UUID, Integer> playerIds = new HashMap<>();
    
    private final HashMap<UUID, HashMap<String, Object>> playerData = new HashMap<>();
    private final HashMap<UUID, HashMap<String, Integer>> playerTickData = new HashMap<>();
    
    public PlayerManager(String prefix, TextFormatting color, DataBank c) 
    {
        super(prefix, color);
        bank = new PlayerBank();
        registerEvent(this);
    }

    public PlayerBank getDataBank() 
    {
        return bank;
    }
    
    public synchronized int getPlayerId(UUID uuid)
    {
        Integer i = playerIds.get(uuid);
        if(i != null)
        {
            return i;
        }
        return bank.getId(uuid);
    }
    
    @SubscribeEvent(priority = EventPriority.HIGH)
    public void onPlayerJoin(PlayerEvent.PlayerLoggedInEvent e)
    {
        EntityPlayer p = e.player;
        if(p == null)
        {
            return;
        }
        
        int id = bank.getId(p.getUniqueID());
        if(id == -1)
        {
            bank.add(p.getUniqueID(), p.getName());
            sendToConsole(p.getName() + " was added.");
            
            id = bank.getId(p.getUniqueID());
            if(id == -1)
            {
                // this should never happen
                sendToConsole(p.getName() + " has no id.");
            }
            else
            {
                playerIds.put(p.getUniqueID(), id);
            }
        }
        else
        {
            playerIds.put(p.getUniqueID(), id);
            bank.changeName(p.getUniqueID(), p.getName());
        }
    }
    
    @SubscribeEvent(priority = EventPriority.LOW)
    public void onPlayerLeaveCleanup(PlayerEvent.PlayerLoggedOutEvent e)
    {      
        UUID uuid = e.player.getUniqueID();
        ScoreboardUtils.remove(uuid);
        playerIds.remove(uuid);
        playerData.remove(uuid);
        playerTickData.remove(uuid);
    }
    
    public void tickPlayerData()
    {
        playerTickData.forEach((uuid, map) -> 
        {
            EntityPlayer p = KajetansMod.server.getPlayerList().getPlayerByUUID(uuid);
            if(p == null)
            {
                return;
            }
            map.entrySet().removeIf(entry -> 
            {
                int time = entry.getValue() - 1;
                if(time <= 0)
                {
                    if(time == 0)
                    {
                        ScriptEvents.onPlayerDataTick(p, entry.getKey());
                    }
                    return true;
                }
                entry.setValue(time);
                return false;
            });
        });
    }
    
    public void setPlayerVar(UUID uuid, String varname, Object value)
    {
        HashMap<String, Object> map = playerData.get(uuid);
        if(map == null)
        {
            map = new HashMap<>();
            playerData.put(uuid, map);
        }
        map.put(varname, value);
    }
    
    public Object getPlayerVar(UUID uuid, String varname)
    {
        HashMap<String, Object> map = playerData.get(uuid);
        if(map == null)
        {
            return null;
        }
        return map.get(varname);
    }
    
    public void setPlayerTimer(UUID uuid, String varname, int time)
    {
        HashMap<String, Integer> map = playerTickData.get(uuid);
        if(map == null)
        {
            map = new HashMap<>();
            playerTickData.put(uuid, map);
        }
        map.put(varname, time);
    }
    
    public int getPlayerTimer(UUID uuid, String varname)
    {
        HashMap<String, Integer> map = playerTickData.get(uuid);
        if(map == null)
        {
            return -1;
        }
        return map.getOrDefault(varname, -1);
    }
    
    public void clearPlayerData(UUID uuid)
    {
        playerTickData.remove(uuid);
        playerData.remove(uuid);
    }
    
    @SubscribeEvent
    public void onPlayerConnection(PlayerConnectionEvent e)
    {      
        ModDimensions.sendNotificationsForDimensions(e.getEntityPlayer());
    }
}
